import { defineStore } from 'pinia'
import { PageNumberPaginationParameters } from '@/api'
import { errorParser } from '@/helpers'
import { UUID } from '@/types'
import { ElementDataset } from '@/types/dataset'
import { useNotificationStore } from '@/stores'
import * as api from '@/api'

interface State {
  /**
   * List of dataset that contains a specific element
   */
  elementDatasets: { [elementId: UUID]: ElementDataset[] }
}

export const useElementsStore = defineStore('elements', {
  state: (): State => ({
    elementDatasets: {}
  }),
  actions: {
    async listElementDatasets (elementId: UUID, params: PageNumberPaginationParameters = {}) {
      // Avoid listing datasets twice for an element
      if ((!params.page || params.page === 1) && this.elementDatasets?.[elementId]) return
      // List datasets containing a specific element through all pages
      try {
        const response = await api.listElementDatasets(elementId, params)

        // Progressively add results to the store
        if (elementId in this.elementDatasets) this.elementDatasets[elementId].push(...response.results)
        else this.elementDatasets[elementId] = response.results

        // Follow pagination without awaiting, until we fetched all the data
        if (response.next !== null) this.listElementDatasets(elementId, { ...params, page: response.number + 1 })
      } catch (err) {
        useNotificationStore().notify({ type: 'error', text: errorParser(err) })
        throw err
      }
    }
  }
})
